Endpoints report shows subjects who have reached the protocol endpoint and should enroll to the DM Referral Schedule.
<BR>
<BR>
Pay attention to the "LAST_UPDATED" column. This tells you the last time the report was updated per subject.
<BR>
<BR>
Unlike other reports, this report does not automatically update but you can update it :
<UL>
    <LI>Place a check mark next to one or more subjects and select 'Regenerate selected report data' from the Action menu.</LI>
</UL>